package de.fhw.data.bag.referenceImpl;

import de.fhw.data.function.Fct;
import de.fhw.data.function.Fct2;
import de.fhw.data.function.*;
import de.fhw.data.map.Map;
import de.fhw.data.map.MapFactory;
import de.fhw.data.bag.Bag;
import de.fhw.data.bag.BagImpl;
import de.fhw.data.tuple.Pair;

public class BagAsMapImpl<E> extends BagImpl<E> {
	
	final MapFactory<E,Integer> mf;
	final Map<E,Integer> m;
	
	BagAsMapImpl(MapFactory<E,Integer> mf, Map<E,Integer> m) {
		this.mf = mf;
		this.m  = m;
		++newBag;
	}
	
	BagAsMapImpl(MapFactory<E,Integer> mf) {
		this(mf,mf.empty());
	}
	
	/**
	 * generate a new empty Bag
	 */
	public Bag<E> empty() {
		if (isEmpty())
			return this;
		return
		new BagAsMapImpl<E>(mf);
	}
	
	/**
	 * reimplement isEmpty for efficiency
	 */
	@Override
	public boolean isEmpty() {
		return
		m.isEmpty();
	}

	public
	int occurences(E e) {
		Integer occ = m.getValue(e);
		if (occ == null)
			return 0;
		return occ;		
	}

	/**
	 * insert implemented via map insert
	 */
	public Bag<E> insert(E e, int occ) {
		if (occ > 0)
			return
			new BagAsMapImpl<E>(mf,m.insertWith(e,occ,Predef.Int.plus));

		if (occ == 0)
			return
			this;

		throw
		new IllegalArgumentException("insert for bags called with negative # of occurences");
	}
	/**
	 * remove implemented via map remove
	 */
	@Override
	public Bag<E> remove(E e, int occ) {
		int occ1 = occurences(e);
		
		if (occ1 == 0)
			return this;
		
		if (occ1 <= occ)
			return
			new BagAsMapImpl<E>(mf,m.remove(e));
		
		return
		new BagAsMapImpl<E>(mf,m.insert(e,occ1 - occ));
	}
	
	/**
	 * implement choose1 via map choose
	 */
	@Override
	protected
	E choose1() {
		Pair<E,Integer> e = m.choose();
		return
		e == null
		? null : e.fst;
		
	}
	/**
	 * implementation of fold via map fold
	 */
	public
	<R>
	R fold(final Fct2<E,R,R> f, R init) {
		Fct3<E,Integer,R,R> fo
		= new Fct3<E,Integer,R,R>() {
			public
			R apply(E k, Integer v, R r) {
				for (int i = 0; i<v; ++i) {
					r = f.apply(k,r);
				}
				return r;
			}
		};
		return m.fold(fo,init);
	}

	// TODO this declaration leads to error: method becomes ambiguous
	/**
	 * Ecipse bug: The method is ambiguous for the type ...
	 * 
	 * If bug is removed, delete this wrapper method
	 */

	public
	<R>
	R foldMap(
			final Fct2<R,R,R> combine,
			final Fct<E,R> map,
			R init) {
		return eclipseBugFoldMap(combine,map,init);
	}

	// TODO this declaration leads to error: method becomes ambiguous
	/**
	 * Ecipse bug: The method is ambiguous for the type ...
	 * 
	 * If bug is removed, delete this wrapper method
	 */
	public
	<R>
	R fold1Map(
			final Fct2<R,R,R> combine,
			final Fct<E,R> map,
			R emptyRes) {
		return eclipseBugFold1Map(combine,map,emptyRes);
	}

	/**
	 * all quantor implemented with fold
	 */
	@Override
	public
	Boolean all(final Fct<E,Boolean> p) {
		Fct2<E,Integer,Boolean> p1 =
			new Fct2<E,Integer,Boolean>() {
			public
			Boolean apply(E e, Integer occ){
				return
				p.apply(e);
			}
		};
		return
		m.all(p1);
	}
	
	/**
	 * exists quantor implemented with fold
	 */
	@Override
	public
	Boolean any(final Fct<E,Boolean> p) {
		Fct2<E,Integer,Boolean> p1 =
			new Fct2<E,Integer,Boolean>() {
			public
			Boolean apply(E e, Integer occ){
				return
				p.apply(e);
			}
		};
		return
		m.any(p1);
	}
	
	/**
	 * inv implemented by map invariant
	 */
	public boolean inv() {
		Fct2<E,Integer,Boolean> isPos =
			new Fct2<E,Integer,Boolean>() {
			public
			Boolean apply(E e, Integer occ) {
				return occ > 0;
			}
		};
		return m.inv() && m.all(isPos);
	}
	
	/**
	 * add map space and wrapper object space
	 */
	public Pair<Integer, Integer> objectsUsed() {
		Pair<Integer,Integer> msp = m.objectsUsed();
		
		int noOfObjects = msp.fst + 1;	// m nodes, 1 wrapper
		int noOfFields  = msp.snd + 2;	// n node fields, 2 wrapper field
		return
		new Pair<Integer, Integer>(noOfObjects, noOfFields);
	}
	
	/**
	 * the static counters for construtor calls for the wrapper and the node objects
	 */
	static int newBag  = 0;
	
	public
	Pair<Integer, Integer> objectsAllocated() {
		int nm = newBag;
		newBag = 0;
		Pair<Integer,Integer> sp = m.objectsAllocated();
		
		return
		new Pair<Integer,Integer>(nm + sp.fst, 2 * nm + sp.snd);
	}
	
	public
	String fingerprint() {
		StringBuffer res = new StringBuffer();
		res.append(this.hashCode());
		res.append(",").append(mf.hashCode());
		res.append(",").append(m.fingerprint());

		return
		res.toString();
	}
	
}
