/**
 * $Id: Comp.java,v 1.1 2006/02/07 19:30:07 uwe Exp $
 */

package de.fhw.data.function;

/**
 * specialisation of Fct2 interface:
 * for comparator functions.
 * <p>
 * These functions must return the Integer values +1,0,-1
 * which reflect the ordering relation for the two values:
 * <code>x1 &gt; x2, x1 == x2, x1 &lt; x2</code>
 * <p>
 * The restriction of the result to +1,0,-1 allows for
 * using 3 way switch statements, like in binary search routines
 * <p>
 * <pre>
 * switch (cmp.apply(x1,x2) {
 *   case -1: // process left subtree
 *            return ...;
 *   case 0:  // process node
 *            return ...;
 *   case +1: // process right subtree
 *            return ...;
 * }
 * </pre>
 * <p>
 * These comparator functions can be used for building
 * ordered containers, e.g. ordered set. The comparator objects
 * can be set by the associated factory methods of the collections 
 * <p>
 * @param <X> the argument type
 * 
 * @see Predef.Int
 * @see Predef.Str
 */

public
interface Comp<X> extends Fct2<X,X,Integer> {
}
