/**
 * $Id: OrderedSetImpl.java,v 1.3 2006/02/24 21:21:03 uwe Exp $
 */
package de.fhw.data.set;

import de.fhw.data.function.*;

/**
 * default implementations for OrderedSet
 *
 */
public abstract
class OrderedSetImpl<E>
extends SetImpl<E> {
	protected final
	Fct2<E,E,Integer> compare;
	
	protected
	OrderedSetImpl(Fct2<E,E,Integer> compare) {
		this.compare = compare;
	}
	
	/**
	 * findMin implementation via fold
	 */
	@Override
	public E findMin() {
		Fct2<E,E,E> minElem =
			new Fct2<E,E,E>() {
			public
			E apply(E e, E m) {
				if (m == null)
					return e;
				if (compare.apply(e,m) == -1)
					return e;
				return m;
			}
		};
		E res = fold(minElem,(E)null);
		if (res == null)
			throw
			new IllegalArgumentException("findMin applied to an empty set");
		return res;
	}
	
	/**
	 * findMax implementation via fold
	 */
	@Override
	public E findMax() {
		Fct2<E,E,E> maxElem =
			new Fct2<E,E,E>() {
			public
			E apply(E e, E m) {
				if (m == null)
					return e;
				if (compare.apply(e,m) == +1)
					return e;
				return m;
			}
		};
		E res = fold(maxElem,(E)null);
		if (res == null)
			throw
			new IllegalArgumentException("findMax applied to an empty set");
		return res;
	}
	/**
	 * remove the minimum element
	 * 
	 * implemented with findMin and remove
	 */
	@Override
	public
	Set <E> removeMin() {
		if (isEmpty())
			return this;
		return
		remove(findMin());
	}
	
	/**
	 * remove the maximum element
	 * 
	 * implemented with findMax and remove
	 */
	@Override
	public
	Set <E> removeMax() {
		if (isEmpty())
			return this;
		return
		remove(findMax());
	}
	
	/**
	 * toString is implemented with findMin, choose and remove
	 * the elements in the string are ordered
	 */
	@Override
	public
	String toString() {
		if (isEmpty())
			return "{}";
		return
		"{"
		+ toString1(this);
	}

	private
	String toString1(Set<E> s) {
		E e = s.findMin();
		Set<E> s1 = s.remove(e);
		return
		e.toString() +
		(s1.isEmpty() ? "}" : ", " + toString1(s1));
	}
}
