/**
 * $Id: SetAsMapFactory.java,v 1.3 2006/01/31 18:27:42 uwe Exp $
 */
package de.fhw.data.set.referenceImpl;

import de.fhw.data.map.MapFactory;
import de.fhw.data.map.referenceImpl.MapAsLinkedListFactory;
import de.fhw.data.set.Set;
import de.fhw.data.set.SetFactory;

/**
 * A concrete factory for set implementations via maps.
 * The attribute value of the map elements is set to THE value.
 * It is ignored during processing.
 * 
 * @param <E> the element type
 */

public
class SetAsMapFactory<E> implements SetFactory<E> {
	final MapFactory<E,Integer> mf;
	/**
	 * 
	 * @param mf the factory for the underlying map implementation.
	 * If this factory is yields an efficient implementation,
	 * the set implementation also becomes rather efficient
	 */
	public SetAsMapFactory(MapFactory<E,Integer> mf) {
		this.mf = mf;
	}
	/**
	 * the default map factory is for the reference implementation
	 * in de.fhw.data.map.referenceImpl, a very inefficient one
	 *
	 */
	public SetAsMapFactory() {
		this(new MapAsLinkedListFactory<E,Integer>());
	}
	/**
	 * generation of an empty set. The result can be used as a prototype.
	 */
	public Set<E> empty() {
		return
		new SetAsMapImpl<E>(mf);
	}
}
