package de.fhw.test;

import de.fhw.data.bag.Bag;
import de.fhw.data.bag.referenceImpl.BagAsMapFactory;
import de.fhw.data.function.Fct;
import de.fhw.data.function.Fct2;
import de.fhw.data.function.Predef;
import de.fhw.data.list.List;
import de.fhw.data.list.referenceImpl.ListAsLinkedListFactory;
import de.fhw.data.map.*;
import de.fhw.data.map.referenceImpl.*;
import de.fhw.data.set.Set;
import de.fhw.data.set.referenceImpl.OrderedSetAsSetFactory;
import de.fhw.data.set.referenceImpl.OrderedSetAsSetFactoryWithComp;
import de.fhw.data.set.referenceImpl.SetAsMapFactory;

/**
 * the 1. map test
 * 
 * @author uwe
 *
 */
public class Test0 {
	
	/**
	 * the prototype for a map implemented as linked list
	 */
	static Map<Integer,Integer> mapPrototype =
		(new MapAsLinkedListFactory<Integer,Integer>())
		.empty();
	
	/**
	 * the prototype for a set implemented as map
	 */
	static Set<Integer> setPrototype =
		(new SetAsMapFactory<Integer>(
				new MapAsLinkedListFactory<Integer,Integer>())
		)
		.empty();
	
	/**
	 * the prototype for the ordered integer set
	 */
	static Set<Integer> orderedSetPrototype =
		(new OrderedSetAsSetFactory<Integer>())
		.empty();
	
	// this auxiliary variable is required by the compiler,
	// when inserting the cmpReverse call below
	// compile errors are issued
	static Fct2<Integer,Integer,Integer> ci = Predef.cmpReverse();
	
	/**
	 * the prototype for reverse ordered integer sets
	 */
	static Set<Integer> reverseOrderedSetPrototype =
		(new OrderedSetAsSetFactoryWithComp<Integer>(ci))
		.empty();

	/**
	 * the prototype for a bag implemented as map
	 */
	static Bag<Integer> bagPrototype =
		(new BagAsMapFactory<Integer>(
				new MapAsLinkedListFactory<Integer,Integer>())
		)
		.empty();
	
	/**
	 * the prototype for a list implemented as linked list
	 */
	static List<Integer> listPrototype =
		(new ListAsLinkedListFactory<Integer>())
		.empty();
	/**
	 * function constants
	 */
	static Fct<Integer,Integer> id =
		Predef.id();

	static Fct2<Set<Integer>,Set<Integer>,Set<Integer>>	union =
		Predef.Sets.union();
	
	public static void main(String[] args) {
		
		Map<Integer,Integer>
		m  = mapPrototype,
		m2 = mapPrototype;
		
		for (int i = 1; i < 100; ++i) {
			m = m.insert(i%10, i*i);
		}
		
		for (int i = 1; i < 10; ++i) {
			m2 = m2.insert(i,i*i);
		}
		String fpm  = m.fingerprint();
		String fpm2 = m2.fingerprint();
		
		System.out.println(m);
		System.out.println(m.objectsUsed());
		System.out.println(m.objectsAllocated());
		
		System.out.println(m2);
		System.out.println(m.union(m2));
		
		Fct2<Integer,Integer,Integer> plus
		= new Fct2<Integer,Integer,Integer>(){
			public Integer apply(Integer v1, Integer v2) {
				return v1 + v2;
			}
		};
		
		System.out.println(m.unionWith(m2,plus));
		
		// same as above, but with predefined plus
		System.out.println(m.unionWith       (m2,Predef.Int.plus));
		System.out.println(m.intersectionWith(m2,Predef.Int.plus));
		
		Set<Integer> s1 = m.keys(setPrototype);
		Set<Integer> s2 = m.values(setPrototype);
		
		System.out.println(fpm.equals(m.fingerprint()));
		System.out.println(fpm2.equals(m2.fingerprint()));
		
		System.out.println(s1.objectsUsed());
		System.out.println(s2.objectsUsed());
		
		System.out.println(s1);
		System.out.println(s2);
		System.out.println(s1.union(s2));
		System.out.println(s1.toList(listPrototype));

		System.out.println(m.objectsAllocated());
		System.out.println(setPrototype.objectsAllocated());
		
		// convert between two set implementations
		
		Set<Integer> os1 = s1.foldMap(
				new Fct2<Set<Integer>,Set<Integer>,Set<Integer>>() {
					public
					Set<Integer> apply(Set<Integer> x1,Set<Integer> x2) {
						return x1.union(x2);
					}
				},
				new Fct<Integer,Set<Integer>>() {
					public
					Set<Integer> apply(Integer x) {
						return orderedSetPrototype.single(x);
					}
				}
				,orderedSetPrototype);
		System.out.println(os1);
		
		// same as os1, but using prelude generators
		
		Fct<Integer,Set<Integer>>
		singleSet = Predef.Sets.singleSet(orderedSetPrototype);
		
		Set<Integer> os1a = s1.foldMap(union,singleSet,orderedSetPrototype);
		System.out.println(os1a);
		
		// simpler variant: use of map
		
		Set<Integer> os2 = s1.map(
				// Prelude.id() does not work (why? See below!)
				new Fct<Integer,Integer>() {
					public
					Integer apply(Integer x) {return x;}
				}
				,orderedSetPrototype);
		System.out.println(os2);
		
		// easy to use variant: here it works: explicit types are required
		
		Set<Integer> os3 = s1.map(id,orderedSetPrototype);
		
		System.out.println(os3);
		System.out.println(os3.map(id));
		
		Set<Integer> osr1 = s1.map(id,reverseOrderedSetPrototype);		
		System.out.println(osr1);
		
		System.out.println(m.objectsAllocated());
		System.out.println(setPrototype.objectsAllocated());
		System.out.println(orderedSetPrototype.objectsAllocated());
		System.out.println(reverseOrderedSetPrototype.objectsAllocated());
		
		Bag<Integer> b0 = bagPrototype;
		for (int i = 1; i < 5; ++i) {
			b0 = b0.insert(i,i);
		}
		String fpb0 = b0.fingerprint();
		
		System.out.println(b0);
		System.out.println(b0.toList(listPrototype));

		System.out.println(fpb0.equals(b0.fingerprint()));

		List<Integer> l0 = listPrototype;
		System.out.println(l0.fingerprint());
		
		for (int i = 0; i < 10; ++i) {
			l0 = l0.cons(i);
		}
		System.out.println(l0.fingerprint());

		System.out.println(l0);
		String fpl0 = l0.fingerprint();

		System.out.println(l0.append(42));
		System.out.println(l0.concat(l0));

		System.out.println(l0.map(id));
				
		List<Integer> l1 = l0.zipWith(Predef.Int.plus,l0);
		System.out.println(l1);
		
		List<Integer> l2 = l1.zipWith(Predef.Int.plus,l1,l0);
		System.out.println(l2);
		
		Integer sum2 = l2.foldl(Predef.Int.plus,0);
		System.out.println(sum2);	

		sum2 = l2.foldr(Predef.Int.plus,0);
		System.out.println(sum2);	

		System.out.println(fpl0.equals(l0.fingerprint()));
		
		System.out.println(l0.objectsUsed());
		System.out.println(l0.objectsAllocated());
	}	
}
